#!/bin/bash
# author           : P.R. faasse (p.faase1@chello.nl) <prf>
# maintainer       : P.R. Faasse
# source location  : www.kde.org
# additional files : none
# depends upon     : Qt, arts, kdelibs
# notes	           : re-enabled arts
# update history   
# 16-aug-2006: 1Z28 -> 2Z28: added the patch needed to support the
#                            'tango' icons standard
# 22-mai-2007: changed mail address, modified for kde-3.5.7 (46.1)
# 08-jul-2007: changed the /etc/X11/dm/Sessions/kde.desktop to match the new ZenWalk 
#              standard for windows-managers (48.1)
# 24-sep-2007: added two patches for security problems (48.2)
# 17-oct-2007: removed patches, going to kde-3.5.8 (50.1)
# 20-oct-2007: added inclusion of the crystalsvg icons that have been
#              removed from kdelibs
# 05-nov-2007: added a patch for the new location of the gdm socket
# 13-nov-2007: removed all svgz-icons from kdebase: they cause trouble with xfce, and seem to have no real function
#              in kde itself. 
# 27-nov-2007: brutal upgrade of the build-number to keep snapshot and
#              current packages separated; chenge in xinitrc.kde to keep
#              kde and gnome menus separated
# 28-nov-2007: udated the kdm config part, the background settings, the
#              XDG_MENU_PREFIX
# 03-mrt-2008: kdmrc config made into sed-'scripts', logo file copied
# 30-apr-2008: matched up kdebase with the kdelibs that addresses the problem 
#              of /var/tmp/kdecache_<user> getting corrupted when running kde
#              applications from xfce.
# 25-aug-2008: version = 3.5.10 build = 64.1
# 26-aug-2008: rebuild, background is a png now, logo included.
# 02-dec-2008: rebuild, gmg pointed to a bug-fix patch; also: removed
#              non-functional src-file generation code
#


#
# what package is going to be built
#
softversion='3.5.10'
softname='kdebase'
kdeversion="$softversion"
packageversion='64.2'

#
# a lonesome hook
#
(

echo "-- start -----"

#
# where is the source of the thing
#
srcurl="ftp://ftp.kde.org/pub/kde/stable/$softversion/src/$softname-$softversion.tar.bz2"

#
# ZenWalk standard: CPU type and architecture, as well as the
# version number of ZenWalk this package is made for (on?).
#
# arch = 'i686' -> the compiler uses only 80486 instructions
# tune = 'i686' -> the optimising is done for a PII and above CPU
#
arch='i686'
cpu='i686'

#
# an extra parameter: The set of packages is part of the build of
# kde-3.5.10. I find little merit in makeing a whole load of diretories
# in /usr/src (destination) for each individual package. I'll put the
# lot of them in /usr/src/kde-3.5.10 instead
#
kdeversion="3.5.10"

#
# the package 'destination'
#
prefix=/usr

#
# building-computer dependent: the number of parallel 'jobs' of 'make'
#
# 'heavy' computer: 6 build-jobs at once
#
#numjobs='-j6'
#
# 'light' computer: 2 build-jobs at once
#
numjobs='-j2'

#
# parameters needed for the build process
#
buildir=$(pwd)
package="$softname-$softversion-$arch-$packageversion"

echo "-- build-time dependencies check -----"

depsok=passed

cd /var/log/packages
 
#
# dependencies check
#
for file in \
  qt-base \
  arts \
  kdelibs \
  cyrus-sasl \
  krb5 \
  dbus-qt3 \
; do
   #
   # Check the dependency
   #
   if [ -f $file-?.* ] ; then 
      echo "package: $file is installed: `ls -d $file*`: good ...";
   else
      echo "package: $file not found: please install a $file package first";
      depsok=fail
   fi
   #
done

cd $buildir
 
echo "Dependencies check result: $depsok"
#
# check if all dependencies are there
#
if [[ $depsok == passed ]]; then
   unset $depsok;
else
   exit 1;
fi

#
# do package-specific settings 
#
# This avoids compiling a version number into KDE's .la files:
QTDIR=/usr/lib/qt
export QTDIR
#
# KDE additions:
#
KDEDIR=$prefix
PATH=$PATH:$KDEDIR/bin
MANPATH=$MANPATH:$KDEDIR/man
export KDEDIR PATH MANPATH

#
# prepare the build result directory
#
dest="$buildir/$package"

rm -rf $dest
#
mkdir -p $dest
mkdir -p $dest/install
mkdir -p $dest/$prefix/bin

#
# the /usr/src part
#
destsrc="$dest/usr/src/kde-$kdeversion"
mkdir -p $destsrc

#
# put slack-desc in place. Either a supplied one, or some default
# text.
#
bbstring=`echo $softname | sed "s/./ /g"`
cat <<EOF > $dest/install/slack-desc
$bbstring|-----handy-ruler------------------------------------------------------|
$softname: Core applications, icons and configuration info for KDE 
$softname:
$softname: kdm, kwin, konqueror, konsole, kicker, kaudio, kdehelp, 
$softname: kthememgr,kcheckpass, kikbd, kscreensaver, kcontrol, kfind,
$softname: kfontmanager, kmenuedit, and kappfinder.
$softname: 
$softname: 
$softname: 
$softname: 
$softname:
$softname:
EOF

#
# add an 'installed' file for /etc/X11/dm/Sessions/kde.desktop 
# this makes KDE show up in the gdm display-manager chooser
#
mkdir -p $dest/etc/X11/dm/Sessions
cat >$dest/etc/X11/dm/Sessions/kde.desktop <<"EOF"
[Desktop Entry]
Encoding=UTF-8
Name=KDE
Comment=The K desktop environment
Exec=/etc/X11/xinit/xinitrc.kde
# no icon yet, only the top three are currently used
Icon=
Type=Application
EOF

#
# set the mode... (is that really needed?) The file looks pretty non-executable to me
#
chmod a+x $dest/etc/X11/dm/Sessions/kde.desktop

#
# option: add some (all?) kde programs to the 'standard' menu: not quite
# now..
#
#------------------------------------------------------------------------------
# the real build process
#------------------------------------------------------------------------------
#
echo "-- unpacking the kdebase source archive -----"

cd $buildir
file="$softname-$softversion.tar.bz2"
srcdir="$(basename $file .tar.bz2)" 
rm -rf $srcdir
tar xf $file
cd $srcdir  
#
# set target permissions
#
chown -R root.root . 
find . -perm 777 -exec chmod 755 {} \;
find . -perm 775 -exec chmod 755 {} \;
find . -perm 666 -exec chmod 644 {} \;
find . -perm 664 -exec chmod 644 {} \;
#
# 16-aug-2006: patch for the 'tango' icons scheme
#
echo "-- patching for the tango icons scheme -----"

   patch -p0 <<"EOF"
Index: kdialog/kdialog.cpp
===================================================================
--- kdialog/kdialog.cpp	(revision 495779)
+++ kdialog/kdialog.cpp	(working copy)
@@ -622,6 +622,20 @@
             context = KIcon::Application;
         else if ( contextStr == QString::fromLatin1( "Actions" ) )
             context = KIcon::Action;
+        else if ( contextStr == QString::fromLatin1( "Animations" ) )
+            context = KIcon::Animation;
+        else if ( contextStr == QString::fromLatin1( "Categories" ) )
+            context = KIcon::Category;
+        else if ( contextStr == QString::fromLatin1( "Emblems" ) )
+            context = KIcon::Emblem;
+        else if ( contextStr == QString::fromLatin1( "Emotes" ) )
+            context = KIcon::Emote;
+        else if ( contextStr == QString::fromLatin1( "International" ) )
+            context = KIcon::International;
+        else if ( contextStr == QString::fromLatin1( "Places" ) )
+            context = KIcon::Place;
+        else if ( contextStr == QString::fromLatin1( "Status" ) )
+            context = KIcon::StatusIcon;
 
 	KIconDialog dlg(0, "icon dialog");
 	kapp->setTopWidget( &dlg );
EOF

echo "-- patching mediamanager/halbackend bug -----"

   patch -p0 <<"EOF"
Index: kioslave/media/mediamanager/halbackend.cpp
===================================================================
--- kioslave/media/mediamanager/halbackend.cpp	(revision 864098)
+++ kioslave/media/mediamanager/halbackend.cpp	(working copy)
@@ -851,13 +851,6 @@
         result << tmp;
     }
 
-	if ( valids.contains("locale") )
-	{
-		value = config.readBoolEntry( "locale", true );
-		tmp = QString( "locale=%1" ).arg( value ? "true" : "false" );
-		result << tmp;
-	}
-
     if (valids.contains("utf8"))
     {
         value = config.readBoolEntry("utf8", true);
@@ -878,6 +871,15 @@
             result << "shortname=lower";
     }
 
+
+    if (valids.contains("locale") && fstype == "ntfs-3g")
+    {
+        char *cType;
+        if ( (cType = getenv("LC_ALL")) || (cType = getenv("LC_CTYPE")) || (cType = getenv("LANG")) ) {
+            result << QString("locale=%1").arg(cType);
+        }
+    }
+
     if (valids.contains("sync"))
     {
         value = config.readBoolEntry("sync", ( valids.contains("flush") && !fstype.endsWith("fat") ) && removable);
@@ -931,7 +933,7 @@
 
     QMap<QString,QString> valids = MediaManagerUtils::splitOptions(options);
 
-    const char *names[] = { "ro", "quiet", "atime", "uid", "utf8", "flush", "sync", "locale", 0 };
+    const char *names[] = { "ro", "quiet", "atime", "uid", "utf8", "flush", "sync", 0 };
     for (int index = 0; names[index]; ++index)
         if (valids.contains(names[index]))
             config.writeEntry(names[index], valids[names[index]] == "true");
@@ -951,10 +953,6 @@
         config.writeEntry("automount", valids["automount"]);
     }
 
-	if (valids.contains("locale") ) {
-		config.writeEntry("locale", valids["locale"]);
-	}
-
     return true;
 }
 
@@ -1153,11 +1151,6 @@
         soptions << QString("uid=%1").arg(getuid());
     }
 
-    if (valids["locale"] == "true")
-	{
-		soptions << QString("locale=%1").arg( KGlobal::locale()->language() ); 
-	}
-
     if (valids["ro"] == "true")
         soptions << "ro";
 
@@ -1182,6 +1175,11 @@
         soptions << QString("shortname=%1").arg(valids["shortname"]);
     }
 
+    if (valids.contains("locale"))
+    {
+        soptions << QString("locale=%1").arg(valids["locale"]);
+    }
+
     if (valids.contains("journaling"))
     {
         QString option = valids["journaling"];
EOF

#
# gdm has a new location for its socket. We have to 'follow'
#
echo "-- patching for the new gdm_socket location -----"
(
 cd kdmlib
 sed -i "s/\/tmp\/\.gdm_socket/\/var\/run\/gdm_socket/" dmctl.cpp
)

echo "-- configure -----"
#
# configure package for compilation
#
export CFLAGS="-O2 -L/usr/lib/qt/lib -march=$arch -mtune=$cpu"  
export CXXFLAGS="-O2 -L/usr/lib/qt/lib -march=$arch -mtune=$cpu"  
./configure \
  --prefix=$prefix \
  --sysconfdir=/etc \
  --localstatedir=/var \
  --mandir=/usr/man \
  --with-shadow \
  --with-xinerama \
  --disable-debug \
  --program-prefix="" \
  --program-suffix="" \
  || exit 1 

#
# make package
#
echo "-- make -----"
make $numjobs || exit 1
#
# install package to the target directory
#
echo "-- make install -----"
make install DESTDIR=$dest || exit 1
#
# add 'default' files
#
echo "-- adding default files -----"

mkdir -p $dest/usr/doc/$srcdir
cp -a \
  AUTHORS COPYING INSTALL NEWS README TODO \
  $dest/usr/doc/$srcdir 
      
echo "-- add 'special files' -----"

#
# copy some files, add the configuration to the kde directory
#
    mkdir -p $dest/etc/X11/xinit $dest/etc/profile.d
    
#
# prepare an xinitrc.kde
#
    cat >$dest/etc/X11/xinit/xinitrc.kde <<"EOF"
#!/bin/sh
# $XConsortium: xinitrc.cpp,v 1.4 91/08/22 11:41:34 rws Exp $

#
# this makes kde use the file /etc/xdg/menus/kde-applications.menu
# for its menu files. Also evades conflicts of kde with gnomce/e17
#

userresources=$HOME/.Xresources
usermodmap=$HOME/.Xmodmap
sysresources=/usr/X11/lib/X11/xinit/.Xresources
sysmodmap=/usr/X11/lib/X11/xinit/.Xmodmap

# merge in defaults and keymaps

if [ -f $sysresources ]; then
    xrdb -merge $sysresources
fi

if [ -f $sysmodmap ]; then
    xmodmap $sysmodmap
fi

if [ -f $userresources ]; then
    xrdb -merge $userresources
fi

if [ -f $usermodmap ]; then
    xmodmap $usermodmap
fi

# Start the window manager:
startkde
EOF

#
# prepare some kde.sh and kde.csh
#
    cat >$dest/etc/profile.d/kde.sh <<"EOF"
#!/bin/sh
# KDE additions:
KDEDIR=/usr
PATH=$PATH:$KDEDIR/bin
MANPATH="${MANPATH}:/usr/man"
export KDEDIR PATH MANPATH
export KDE_MENU_PREFIX="kde-"
EOF

#
# warning: the --prefix is hard-coded here: 
# in this line:  >>>> setenv KDEDIR /usr <<<<
#
    cat >$dest/etc/profile.d/kde.csh <<"EOF"
#!/bin/csh
# KDE additions:
if ( ! $?KDEDIR ) then
    setenv KDEDIR /usr
endif
if ( $?PATH ) then
    setenv PATH $KDEDIR/bin:$PATH
else
    setenv PATH $KDEDIR/bin
endif
if ( $?MANPATH ) then
    setenv MANPATH ${MANPATH}:$KDEDIR/man
else
    setenv MANPATH $KDEDIR/man
endif
if ( ! $?KDE_MENU_PREFIX ) then
    setenv KDE_MENU_PREFIX kde-
endif
EOF

chmod 755 $dest/etc/X11/xinit/xinitrc.kde \
      $dest/etc/profile.d/kde.sh \
      $dest/etc/profile.d/kde.csh
    
#
# Change the konsole defaults to look like the real Linux console:
#
cat >$dest/$KDEDIR/share/config/konsolerc <<"EOF"
[Desktop Entry]
ActiveSession=0
DefaultSession=shell.desktop
DynamicTabHide=0
Fullscreen=false
Height 768=520
TabViewMode=0
Width 1024=744
bellmode=0
class=konsole-mainwindow#1
defaultfont=Bitstream Vera Sans Mono,12,-1,5,75,0,0,0,0,0
font=8
history=1000
historyenabled=true
keytab=default
schema=Linux.schema
scrollbar=2
tabbar=1
EOF

#
# Not allowing root to login at the console is just plain silly.  If they have physical access to
# the machine, what's the point of trying to stop this?  Even the kdmrc file says this is supposed
# to be the default.
#
sed -i "s|AllowRootLogin=false|AllowRootLogin=true|" \
   $dest/$KDEDIR/share/config/kdm/kdmrc 
#

echo "-- pre-packaging configuration -----"

#
# a small patch to the permissions: if /usr/bin/kcheckpass does not have 
# permissions 6755 then a screen-lock is more or less a one-time operation:
# the unlock process password check requires the capability to run as
# 'root', or it will not be able to unlock the screen
#
cat <<EOF >>$dest/install/doinst.sh
#
# patch the permissions of /usr/bin/kcheckpass
#
chmod 6755 usr/bin/kcheckpass
EOF

#
# remove old kdecache files: they will make the user end up with
# an empty menu. We've moved the xdg from /usr/etc (which is nonsense)
# to the 'canonical' /etc/xdg; old-time kde-users will however have a
# cache file that still contains the old settings. The cache-file will
# be re-created when a new kde-session is started. 
#
cat <<EOF >>$dest/install/doinst.sh
#
# clean up some old kde cache files: i've moved the xdg directory, and
# do not want any existing user-config to refer to the old directories
#
rm -rf var/tmp/kdecache-*
#
# workaround for 'PreselectUser=None' bug
#
rm -f var/lib/kdm/kdmsts
#
EOF

#
# ever since my little 'discussion' (forum really..) with Son|c, i'm inclined
# to add a 'ldconfig' to the end of the doinst.sh file
#
cat <<EOF >>$dest/install/doinst.sh
#
# installpkg does not do this right...
#
ldconfig
EOF
#
# adapt the deafult kdm config files to match the zenwalk setup.
#
(
 cd $dest/usr/share/config/kdm
 #
 # change quite a load of settings of kdm. They are needed to make kdm:
 #
 # - find the default zenwalk xxx.desktop settings-definitions files
 # - enable the background settings: we want the zenwalk background
 # - clean up the looks of kdm, to be any competition at all for gdm....
 # 
 # i prefer to use a few sed statements: more clear as to what we want
 # done
 #
 sed -i "s|#SessionsDirs=/etc/X11/sessions|SessionsDirs=/etc/X11/dm/Sessions|" kdmrc
 #
 # we have our own logo now, previous default was Clock
 #
 sed -i "s|LogoArea=Clock|LogoArea=Logo|" kdmrc
 #
 sed -i "s|#AntiAliasing=true|AntiAliasing=true|" kdmrc
 sed -i "s|#GreetString=K Desktop Environment (%n)|GreetString=Welcome to %h|" kdmrc
 sed -i "s|#UserList=false|UserList=false|" kdmrc
 sed -i "s|PreselectUser=Previous|PreselectUser=None|" kdmrc
 sed -i "s|#PreselectUser=Previous|PreselectUser=None|" kdmrc
 sed -i "s|#BackgroundCfg=|BackgroundCFG=/usr/share/config/kdm/backgroundrc|" kdmrc
 sed -i "s|#UseTheme=true|UseTheme=false|" kdmrc
 #
 # zenwalk-logo, just in case someone prefers no clock
 #
 sed -i "s|LogoPixmap=/usr/share/apps/kdm/pics/kdelogo.png|LogoPixmap=/usr/share/config/kdm/zenwalk.png|" kdmrc
 cp $buildir/zenwalk.png .
 #
 # make sure that the logo is distributed: someone 'else' may want to rebuild kdebase...
 #
 cp $buildir/zenwalk.png $dest/usr/src/kde-$kdeversion/
 #
 # the background config
 #
 cat <<"EOF" >backgroundrc
[Desktop0]
BackgroundMode=Flat
BlendBalance=100
BlendMode=NoBlending
ChangeInterval=60
Color1=0,0,200
Color2=192,192,192
CurrentWallpaper=0
LastChange=0
MinOptimizationDepth=1
MultiWallpaperMode=NoMulti
Pattern=fish
Program=
ReverseBlending=false
UseSHM=false
Wallpaper=default.png
WallpaperList=
WallpaperMode=Scaled
EOF
# sed -i "s|Wallpaper=*|Wallpaper=default.png|" backgroundrc

)

#
# the icon for the games/kids desktop directory is 'wrong'
#
(
 cd $dest/usr/share/desktop-directories
 sed -i "s|Icon=package_games_kids|Icon=package_toys|" kde-games-kids.directory
)

#
# change all svgz icons into svg ones...
# xfce seems to get 'the hickups' from
# svgz icons
#
echo "-- changing icons svgz --> svg -----"
(

 cd $dest
 for file in `find | grep -e "svgz$" | grep "usr/share/icons"`; do
    #
    dd="$(dirname $file)"
    ff="$(basename $file .svgz)"
    # 
    # the 'blunt axe' is to rm them...
    # this seems to do no harm ;-)
    #
    #rm $file

    #
    # un-gzip them and make them into svg
    #
    mv $dd/$ff.svgz $dd/$ff.svg.gz
    gunzip $dd/$ff.svg.gz

 done
)

echo "-- add package-rules files -----"

#
# add some package-rules files
#
cp "$buildir/build-$softname.sh" \
   "$destsrc/build-$softname.sh"

#
# a shell-scrip is more usable when it's executable
#
chmod a+x "$destsrc/build-$softname.sh"

#
# generate the 'get-the source' file
#
cat <<EOF >"$destsrc/getsrc-$softname.sh"
#
# get-the-source script
#
wget -c $srcurl
EOF

#
# a shell-scrip is more usable when it's executable
#
chmod a+x "$destsrc/getsrc-$softname.sh"

#
# post-build processing: 
# - strip executables 
# - compress man-pages
#
echo "-- pre-packaging configuration -----"

( cd $dest
  find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
  find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
  for mandir in $(find . -name man -type d) ; do
    if [ -d $mandir ]; then
      ( cd $mandir
        find . -type f -name "*.?" -exec gzip -9 {} \;
      )
    fi
  done
)

#
# real packaging work: 
#
echo "-- set target ownerships -----"

chown -R root.root $dest/$prefix

echo "-- packaging -----"

cd $dest
makepkg -p -l y -c n $dest.txz
(
 echo "-- generating md5 and dep file -----"
 cd $buildir
 md5sum $package.txz >$package.md5
 #
 echo "-- generating the package dependencies -----"
 ADD="krb5" requiredbuilder -z $package.txz >$package.dep
 #
)

#
# the other end of the lonesome hook
#
) 2>&1 | tee "$softname-$softversion-info.txt"
